/*
 * Copyright (C) 2021. Uber Technologies
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.uber.rib.compose.root.main

import android.widget.FrameLayout
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.viewinterop.AndroidView
import com.uber.rib.compose.R

@Composable
fun MainView(childContent: MainRouter.ChildContent) {
  Column(
    horizontalAlignment = Alignment.CenterHorizontally,
    verticalArrangement = Arrangement.Top,
    modifier = Modifier
      .fillMaxSize()
      .padding(all = 4.dp)
      .padding(top = 14.dp)
      .background(Color(0xFFFFA500))
  ) {
    Text("Main RIB (Compose w/ CompView)")
    Box(
      modifier = Modifier
        .fillMaxWidth()
        .weight(1.0f)
        .padding(4.dp)
        .background(Color.Yellow)
    ) {
      if (childContent.fullScreenContent != null) {
        childContent.fullScreenContent?.invoke()
      } else {
        AndroidView(
          modifier = Modifier.fillMaxSize(), // Occupy the max size in the Compose UI tree
          factory = { context ->
            FrameLayout(context).apply {
              id = R.id.login_logout_container
              setBackgroundColor(android.graphics.Color.DKGRAY)
            }
          }
        )
      }
    }
  }
}

@Preview
@Composable
fun MainViewPreview() {
  MainView(MainRouter.ChildContent())
}
